from guizero import Drawing
from Box2D import *
from math import cos,sin,pi
from .thing import *

class ebPolygon(ebThing):
    def __init__(self, box, *args, **kwargs):
        world  = box.world
        canvas = box.canvas

        # mémorisation de la box
        self.box   = box

        # paramètres enisebox
        self._type      = kwargs.pop('type', 'dynamic')

        # paramètres body
        angle           = kwargs.pop('angle', 0)
        fixedRot        = kwargs.pop('fixedRotation', False)
        bullet          = kwargs.pop('bullet', False)

        # paramètres fixture
        dens            = kwargs.pop('density', 1)
        fric            = kwargs.pop('friction', 0)
        rest            = kwargs.pop('restitution', 0)

        # args est censé contenir la liste des sommets
        verts = args[0]
        # on recalcule tous les sommets dans le repère physique
        verts = [(v[0]/box.scale, box._height-v[1]/box.scale) for v in verts]
        # isobarycentre du polygone, centre du body
        (x, y) = self._poly_center(verts)
        # on recalcule tous les sommets par rapport au centre
        verts = [(v[0]-x, v[1]-y) for v in verts]
        # on applique la rotation par rapport au centre
        # verts = [self._vert_rotate(vert, angle) for vert in verts]
        # body box2d
        if self._type=='static':
            constructor = world.CreateStaticBody
        elif self._type=='kinematic':
            constructor = world.CreateKinematicBody
        else:
            constructor = world.CreateDynamicBody
        # on place le body sur l'isobarycentre du polygone
        body = constructor(
            position = (x,y),
            angle = angle,
            fixedRotation=fixedRot,
            bullet=bullet,
        )
        # si les sommets ne sont pas listés dans l'ordre trigo
        # on retourne la liste
        # if self._poly_area(verts)<0:
        #     pverts.reverse()

        # creation du polygone
        body.CreatePolygonFixture(
            vertices=verts,
            density=dens,
            friction=fric,
            restitution=rest,
        )
        # on garde le lien
        self.body = body
        body.userData = self

        # item canvas
        shape  = body.fixtures[0].shape
        coords = [ body.transform * v for v in shape.vertices ]
        coords = [(c[0]*box.scale, (box._height - c[1])*box.scale) for c in coords]
        self.item = canvas.create_polygon(coords, **kwargs)

    def _poly_center(self, verts):
        n  = len(verts)
        xG = 0
        yG = 0
        for i in range(n):
            xG += verts[i][0]
            yG += verts[i][1]
        return xG/n, yG/n

    def _poly_area(self, verts):
        """
        Return area of a simple (ie. non-self-intersecting) polygon.
        Will be negative for counterclockwise winding.
        """
        accum = 0.0
        for i in range(len(verts)):
            j = (i + 1) % len(verts)
            accum += verts[j][0] * verts[i][1] - verts[i][0] * verts[j][1]
        return accum / 2

    def _vert_rotate(self, vert, alpha):
        new = (
                    cos(alpha)*vert[0]-sin(alpha)*vert[1],
                    sin(alpha)*vert[0]+cos(alpha)*vert[1]
                )
        return new

    def update_item(self):
        box    = self.box
        body   = self.body
        canvas = box.canvas
        shape  = body.fixtures[0].shape
        coords = [ body.transform * v for v in shape.vertices ]
        new    = []
        for c in coords:
            new += box.toCanvasCoords(*c)
        canvas.coords(self.item, new)


from guizero import Drawing
from Box2D import *
from .thing import *

class ebRectangle(ebThing):
    def __init__(self, box, *args, **kwargs):
        world  = box.world
        canvas = box.canvas

        # mémorisation de la box
        self.box   = box

        # paramètres enisebox
        self._type      = kwargs.pop('type', 'dynamic')

        # paramètres body
        angle           = kwargs.pop('angle', 0)
        fixedRot        = kwargs.pop('fixedRotation', False)
        bullet          = kwargs.pop('bullet', False)

        # paramètres fixture
        dens            = kwargs.pop('density', 1)
        fric            = kwargs.pop('friction', 0)
        rest            = kwargs.pop('restitution', 0)

        if len(args)==4:
            x0, y0, x1, y1 = args

        width  = (x1-x0)/box.scale
        height = (y1-y0)/box.scale

        # body box2d
        if self._type=='static':
            constructor = world.CreateStaticBody
        elif self._type=='kinematic':
            constructor = world.CreateKinematicBody
        else:
            constructor = world.CreateDynamicBody
        x,y = (    min(x0,x1)/box.scale+width/2,
                            box._height-max(y0,y1)/box.scale+height/2  )
        body = constructor(
            position = (x,y),
            angle = angle,
            fixedRotation=fixedRot,
            bullet=bullet,
        )
        body.CreatePolygonFixture(
            # box=(width/2, height/2),
            box=(width/2, height/2, (0,0), 0),
            density=dens,
            friction=fric,
            restitution=rest,
        )
        # on garde le lien
        self.body = body
        body.userData = self

        # item canvas
        shape  = body.fixtures[0].shape
        coords = [ body.transform * v for v in shape.vertices ]
        coords = [(c[0]*box.scale, (box._height - c[1])*box.scale) for c in coords]
        self.item = canvas.create_polygon(coords, **kwargs)

    def update_item(self):
        box    = self.box
        body   = self.body
        canvas = box.canvas
        shape  = body.fixtures[0].shape
        coords = [ body.transform * v for v in shape.vertices ]
        new    = []
        for c in coords:
            new += box.toCanvasCoords(*c)
        canvas.coords(self.item, new)


from Box2D import *
from math import pi

class ebThing():
    def __init__(self, *args, **kwargs):
        pass

    def bbox(self):
        fixture = self.body.fixtures[0]
        bbox    = fixture.GetAABB(0)
        (x1, y2) = self.box.toCanvasCoords(*bbox.lowerBound)
        (x2, y1) = self.box.toCanvasCoords(*bbox.upperBound)
        return (x1, y1, x2, y2)


    def ApplyLinearImpulse(self, a, b):
        body = self.body
        body.ApplyLinearImpulse(
                impulse=b2Vec2(a,-b),
                point=body.position,
                wake=True,
        )

    def ApplyAngularImpulse(self, imp):
        body = self.body
        self.body.ApplyAngularImpulse(
                impulse=imp,
                wake=True,
        )

    def ApplyForce(self, a, b):
        body = self.body
        body.ApplyForce(
                force=b2Vec2(a,-b),
                point=body.position,
                wake=True,
        )

    def ApplyTorque(self, torque):
        body = self.body
        body.ApplyTorque(
                torque=torque,
                wake=True,
        )

    @property
    def boxCenter(self):
        # centre de gravité dans le repère de la boîte
        return self.body.worldCenter

    @property
    def canvasCenter(self):
        # centre de gravité dans le repère du canvas
        return self.box.toCanvasCoords(*self.body.worldCenter)

    @property
    def position(self):
        # centre géométrique dans le repère de la boîte
        return self.body.position

    @position.setter
    def position(self, pos):
        # centre géométrique dans le repère de la boîte
        self.body.position = pos

    @property
    def canvasPosition(self):
        # centre géométrique dans le repère du canvas
        return self.box.toCanvasCoords(*self.body.position)

    @canvasPosition.setter
    def canvasPosition(self, pos):
        # centre géométrique dans le repère du canvas
        self.body.position = self.box.toBoxCoords(*pos)

    @property
    def angle(self):
        # angle en radian
        return self.body.transform.angle

    @angle.setter
    def angle(self, value):
        self.body.angle = value

    @property
    def angularDamping(self):
        return self.body.angularDamping

    @angularDamping.setter
    def angularDamping(self, damping):
        self.body.angularDamping = damping

    @property
    def linearDamping(self):
        return self.body.linearDamping

    @linearDamping.setter
    def linearDamping(self, damping):
        self.body.linearDamping = damping

    @property
    def angularVelocity(self):
        return self.body.angularVelocity

    @angularVelocity.setter
    def angularVelocity(self, velocity):
        self.body.angularVelocity = velocity

    @property
    def linearVelocity(self):
        return self.body.linearVelocity

    @linearVelocity.setter
    def linearVelocity(self, velocity):
        v = (velocity[0],-velocity[1])
        self.body.linearVelocity = v

    @property
    def groupIndex(self):
        return self.body.fixtures[0].filterData.groupIndex

    @groupIndex.setter
    def groupIndex(self, index):
        self.body.fixtures[0].filterData.groupIndex = index

    @property
    def categoryBits(self):
        return self.body.fixtures[0].filterData.categoryBits

    @categoryBits.setter
    def categoryBits(self, bits):
        self.body.fixtures[0].filterData.categoryBits = bits

    @property
    def maskBits(self):
        return self.body.fixtures[0].filterData.maskBits

    @maskBits.setter
    def maskBits(self, bits):
        self.body.fixtures[0].filterData.maskBits = bits

    @property
    def fixedRotation(self):
        return self.body.fixedRotation

    @fixedRotation.setter
    def fixedRotation(self, value):
        self.body.fixedRotation = value

    @property
    def bullet(self):
        return self.body.bullet

    @bullet.setter
    def bullet(self, value):
        self.body.bullet = value

    ## propriétés et méthodes liées à l'affichage (item du canvas)

    def gettags(self):
        return self.box.canvas.gettags(self.item)

    def dtag(self, tag):
        self.box.canvas.dtag(self.item, tag)

    def addtag(self, tag):
        self.box.canvas.addtag_withtag(tag, self.item)

    def itemconfigure(self, **kwargs):
        item = self.item
        if kwargs:
            self.box.canvas.itemconfigure(item, **kwargs)
        else:
            return self.box.canvas.itemconfigure(item)

    def itemcget(self, option):
        item = self.item
        return self.box.canvas.itemcget(item, option)



from guizero import Drawing
from PIL import Image, ImageTk
from Box2D import *
from .thing import *
from math import pi,sqrt
from time import time

class ebImage(ebThing):
    def __init__(self, box, x, y, width, height, **kwargs):
        world  = box.world
        canvas = box.canvas

        # mémorisation de la box
        self.box   = box

        # paramètres
        self._type      = kwargs.pop('type', 'dynamic')
        self._file      = kwargs.pop('image', None)

        # paramètres body
        angle           = kwargs.pop('angle', 0)
        fixedRot        = kwargs.pop('fixedRotation', False)
        self._noRot     = kwargs.pop('noRotation', False)
        bullet          = kwargs.pop('bullet', False)

        # paramètres fixture
        dens            = kwargs.pop('density', 1)
        fric            = kwargs.pop('friction', 0)
        rest            = kwargs.pop('restitution', 0)
        imShape         = kwargs.pop('imShape', 'rectangle')

        # body box2d
        if self._type=='static':
            constructor = world.CreateStaticBody
        elif self._type=='kinematic':
            constructor = world.CreateKinematicBody
        else:
            constructor = world.CreateDynamicBody
        body = constructor(
            position = (x/box.scale, box._height-y/box.scale),
            angle=angle,
            fixedRotation=fixedRot,
            bullet=bullet,
        )

        # on enregistre le lien avec le body
        self.body     = body
        body.userData = self

        # fixture et shape box2d
        if imShape=='rectangle':
            # shape rectangle par défaut
            body.CreatePolygonFixture(
                box=(width/box.scale/2, height/box.scale/2, (0,0), 0),
                density=dens,
                friction=fric,
                restitution=rest,
            )
        elif imShape=='circle':
            # shape circle
            r = max(width/box.scale/2, height/box.scale/2)
            body.CreateCircleFixture(
                radius=r,
                density=dens,
                friction=fric,
                restitution=rest,
            )

        # lecture des dimensions de l'image et calcul du ratio
        if self._file:
            with Image.open(self._file) as img:
                w, h        = img.size
                ratio       = (width/w, height/h)
                self._ratio = ratio

        # item canvas
        self.item   = None
        if self._file and not self._noRot:
            # l'image est censée tourner
            # ouverture de l'image
            with Image.open(self._file) as img:
                # on étend l'image avec des pixels transparents
                # pour pouvoir la tourner sans perdre de pixels
                side  = round(sqrt(w**2+h**2))
                new = Image.new('RGBA', (side, side), (0, 0, 0, 0))
                new.paste(  img,
                            (round((side-w)/2), round((side-h)/2)),
                            img.convert('RGBA')
                )
            # nouvelles dimensions
            img  = new
            w, h = img.size
            # stockage de l'image
            self._img   = []
            # top départ
            t0          = time()
            # on tourne l'image par pas de 1°
            for angle in range(360):
                # identifiant de l'image
                imageId = self._file + '-' + str(width) + '-' + str(height) + '-' + str(angle)
                # si l'image tournée a déjà été calculée on la retrouve
                if imageId in box._images.keys():
                    self._img.append(box._images[imageId])
                # sinon on la calcule
                else:
                    # rotation
                    new = img.rotate(angle)
                    # redimensionnement
                    # c'est plus long de le faire ici, mais le résultat est meilleur...
                    new = new.resize(
                            (round(w*ratio[0]),round(h*ratio[1])),
                            # Image.NEAREST,
                            # Image.BICUBIC,
                            Image.LANCZOS,
                    )
                    # stockage
                    pimg = ImageTk.PhotoImage(new)
                    box._images[imageId] = pimg
                    self._img.append(pimg)
            # temps écoulé
            elapsed = round(100*(time()-t0))/100
            # affichage du temps écoulé
            if elapsed:
                print(  '{} : {}s'.format(self._file,
                        str(round(100*(time()-t0))/100))
                    )
            # placement de l'image dans le Canvas
            pos         = body.position
            angle       = round(body.transform.angle*180/pi)
            self.item   = canvas.create_image(
                                pos[0]*box.scale,
                                (box._height - pos[1])*box.scale,
                                image=self._img[angle],
                                **kwargs,
                    )
        elif self._file and self._noRot:
            # l'image est bloquée en rotation
            self.setImage(self._file, **kwargs)

    @property
    def angle(self):
        # angle en radian
        return self.body.transform.angle

    @angle.setter
    def angle(self, value):
        self.body.angle = value
        if self._noRot:
            # image dont l'affichage ne suit pas
            # la rotation de l'objet physique
            self.setImage(self._file)
        else:
            # image dont l'affichage suit
            # la rotation de l'objet physique
            canvas = self.box.canvas
            canvas.itemconfigure(self.item, image=self._img[angle])

    def setImage(self, file, **kwargs):
        self._file = file
        # recharge l'image si elle est bloquée en rotation uniquement
        if self._noRot:
            self._img   = []
            # on récupère d'abord ce dont on a besoin
            ratio  = self._ratio
            body   = self.body
            box    = self.box
            canvas = box.canvas
            angle  = round(body.transform.angle*180/pi)
            # identifiant de l'image
            imageId = file + '-' + str(angle)
            # si l'image tournée a déjà été calculée on la retrouve
            if imageId in box._images.keys():
                self._img.append(box._images[imageId])
            # sinon on la calcule
            else:
                # agrandissement de l'image
                with Image.open(file) as img:
                    w, h   = img.size
                    side   = round(sqrt(w**2+h**2))
                    # on étend l'image avec des pixels transparents
                    # pour pouvoir la tourner sans perdre de pixels
                    new = Image.new('RGBA', (side, side), (0, 0, 0, 0))
                    new.paste(  img,
                                (round((side-w)/2), round((side-h)/2)),
                                img.convert('RGBA')
                    )
                # nouvelles dimensions
                w, h   = new.size
                # rotation de l'image
                new    = new.rotate(angle)
                # redimensionnement
                new    = new.resize(
                                (round(w*ratio[0]),round(h*ratio[1])),
                                # Image.NEAREST,
                                # Image.BICUBIC,
                                Image.LANCZOS,
                        )
                # stockage de l'image
                pimg = ImageTk.PhotoImage(new)
                box._images[imageId] = pimg
                self._img.append(pimg)
            # destruction de l'ancien item du Canvas
            canvas.delete(self.item)
            # placement de l'image dans le Canvas
            pos    = body.position
            item   = canvas.create_image(
                            pos[0]*box.scale,
                            (box._height - pos[1])*box.scale,
                            image=self._img[0],
                            **kwargs,
                        )
            # on mémorise le nouvel item
            self.item     = item

    def update_item(self):
        box    = self.box
        body   = self.body
        canvas = box.canvas
        pos    = body.position
        shape  = body.fixtures[0].shape
        angle  = round(body.transform.angle*180/pi)
        # rotation de l'image si nécessaire
        if not self._noRot:
            canvas.itemconfigure(self.item, image=self._img[angle])
        # déplacement de l'image
        canvas.coords(self.item, box.toCanvasCoords(*pos))



def poly_area(verts):
    """
    Return area of a simple (ie. non-self-intersecting) polygon.
    Will be negative for counterclockwise winding.
    """
    accum = 0.0
    for i in range(len(verts)):
        j = (i + 1) % len(verts)
        accum += verts[j][0] * verts[i][1] - verts[i][0] * verts[j][1]
    return accum / 2

def poly_center(verts):
    n  = len(verts)
    xG = 0
    yG = 0
    for i in range(n):
        xG += verts[i][0]
        yG += verts[i][1]
    return xG/n, yG/n


verts = [(200,200),(100,300),(200,400),(250,300),(200,100)]


print(poly_area(verts))
print(poly_center(verts))


from guizero import App, Drawing
from PIL import Image, ImageTk
from Box2D import *
from random import randint

from .rectangle import ebRectangle
from .circle import ebCircle
from .image import ebImage
from .polygon import ebPolygon

class enisebox():
    def __init__(self, canvas, *args, **kwargs):
        # paramètres pour le canvas
        canvas.update()
        self.scale     = kwargs.pop('scale', 20)
        self.canvas    = canvas
        self._width     = canvas.winfo_width()/self.scale
        self._height    = canvas.winfo_height()/self.scale
        # contactListener
        self._contact   = kwargs.pop('contact', None)
        if self._contact is not None:
            contactListener = ebContactListener(self._contact)
            kwargs['contactListener'] = contactListener
        # le monde box2d
        self.world     = b2World(**kwargs)
        # les images
        self._images  = {}
        # les objets du monde
        self.things   = []

    def create_rectangle(self, *args, **kwargs):
        obj = ebRectangle(self, *args, **kwargs)
        self.things.append(obj)
        return obj

    def create_polygon(self, *args, **kwargs):
        obj = ebPolygon(self, *args, **kwargs)
        self.things.append(obj)
        return obj

    def create_circle(self, *args, **kwargs):
        obj = ebCircle(self, *args, **kwargs)
        self.things.append(obj)
        return obj

    def create_image(self, *args, **kwargs):
        obj = ebImage(self, *args, **kwargs)
        self.things.append(obj)
        return obj

    # obsolète, présent pour compatibilité
    def itemconfigure(self, thing, **kwargs):
        item = thing.item
        if kwargs:
            self.canvas.itemconfigure(item, **kwargs)
        else:
            return self.canvas.itemconfigure(item)

    def find_withtag(self, tagOrId):
        ids = self.canvas.find_withtag(tagOrId)
        return [ thing for thing in self.things if thing.item in ids ]

    def Step(self, *args):
        self.world.Step(*args)
        for obj in self.things:
            obj.update_item()

    def CreateDistanceJoint(self, thingA, thingB, anchorA, anchorB, **kwargs):
        return self.world.CreateDistanceJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            anchorA=self.toBoxCoords(*anchorA),
            anchorB=self.toBoxCoords(*anchorB),
            **kwargs,
        )

    def CreateRevoluteJoint(self, thingA, thingB, anchor, **kwargs):
        return self.world.CreateRevoluteJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            anchor=self.toBoxCoords(*anchor),
            **kwargs,
        )

    def CreatePrismaticJoint(self, thingA, thingB, anchor, axis, **kwargs):
        lowerTranslation = kwargs.pop('lowerTranslation', 0)
        upperTranslation = kwargs.pop('upperTranslation', 0)
        if lowerTranslation:
            kwargs['lowerTranslation']=lowerTranslation/self.scale
        if upperTranslation:
            kwargs['upperTranslation']=upperTranslation/self.scale
        return self.world.CreatePrismaticJoint(
            bodyA=thingA.body,
            bodyB=thingB.body,
            anchor=self.toBoxCoords(*anchor),
            axis=axis,
            **kwargs,
        )

    # def CreateWheelJoint(self, thingA, thingB, **kwargs):
    #     return self.world.CreateWheelJoint(
    #         bodyA=thingA.body,
    #         bodyB=thingB.body,
    #         **kwargs,
    #     )
    #
    # def CreateRopeJoint(self, thingA, thingB, **kwargs):
    #     return self.world.CreateRopeJoint(
    #         bodyA=thingA.body,
    #         bodyB=thingB.body,
    #         **kwargs,
    #     )
    #
    # def CreateFrictionJoint(self, thingA, thingB, **kwargs):
    #     return self.world.CreateFrictionJoint(
    #         bodyA=thingA.body,
    #         bodyB=thingB.body,
    #         **kwargs,
    #     )
    #
    # def CreateGearJoint(self, thingA, thingB, **kwargs):
    #     return self.world.CreateGearJoint(
    #         bodyA=thingA.body,
    #         bodyB=thingB.body,
    #         **kwargs,
    #     )
    #
    # def CreateMouseJoint(self, thingA, thingB, **kwargs):
    #     return self.world.CreateMouseJoint(
    #         bodyA=thingA.body,
    #         bodyB=thingB.body,
    #         **kwargs,
    #     )
    #
    # def CreatePulleyJoint(self, thingA, thingB, **kwargs):
    #     return self.world.CreatePulleyJoint(
    #         bodyA=thingA.body,
    #         bodyB=thingB.body,
    #         **kwargs,
    #     )
    #
    # def CreateWeldJoint(self, thingA, thingB, **kwargs):
    #     return self.world.CreateWeldJoint(
    #         bodyA=thingA.body,
    #         bodyB=thingB.body,
    #         **kwargs,
    #     )

    def DestroyThing(self, thing):
        if thing in self.things:
            self.things.remove(thing)
            self.canvas.delete(thing.item)
            self.world.DestroyBody(thing.body)
            thing.body = None
        else:
            print("enisebox .DestroyThing : vous essayez de détruire un objet qui n'existe pas !")

    def DestroyJoint(self, joint):
        self.world.DestroyJoint(joint)

    def toBoxCoords(self, x, y):
        return (x/self.scale, (self._height*self.scale-y)/self.scale)

    def toCanvasCoords(self, x, y):
        return (int(x*self.scale), int((self._height-y)*self.scale))

    def find_overlapping2(self, bbox, callback):
        # obsolète
        (x1, y1, x2, y2) = bbox
        queryCallback = ebQueryCallback2(callback)
        aabb = b2AABB(
            lowerBound=self.toBoxCoords(x1,y2),
            upperBound=self.toBoxCoords(x2,y1),
        )
        self.world.QueryAABB(queryCallback, aabb)
    #
    # def find_overlapping3(self, x1, y1, x2, y2):
    #     # obsolète
    #     queryCallback = ebQueryCallback()
    #     aabb = b2AABB(
    #         lowerBound=self.toBoxCoords(x1,y2),
    #         upperBound=self.toBoxCoords(x2,y1),
    #     )
    #     self.world.QueryAABB(queryCallback, aabb)
    #     return queryCallback.objects

    def find_overlapping(self, x1, y1, x2, y2):
        objects = []
        for obj in self.things:
            objBbox = self.bbox(obj)
            if self._intersect(*objBbox, x1, y1, x2, y2):
                objects.append(obj)
        return objects

    def _inside(self,x,y,x1,y1,x2,y2):
        if x1<=x<=x2 and y1<=y<=y2:
            return True
        else:
            return False

    def _intersect(self,x1,y1,x2,y2,a1,b1,a2,b2):
        if ( self._inside(x1,y1,a1,b1,a2,b2) or self._inside(x2,y2,a1,b1,a2,b2)
                or self._inside(a1,b1,x1,y1,x2,y2) or self._inside(a2,b2,x1,y1,x2,y2) ):
            return True
        else:
            return False

    @property
    def gravity(self):
        # gravité en m/s/s
        return self.world.gravity

    @gravity.setter
    def gravity(self, value):
        self.world.gravity = value

    def bbox(self, thing):
        fixture = thing.body.fixtures[0]
        bbox = fixture.GetAABB(0)
        (x1, y2) = self.toCanvasCoords(*bbox.lowerBound)
        (x2, y1) = self.toCanvasCoords(*bbox.upperBound)
        return (x1, y1, x2, y2)

class ebContactListener(b2ContactListener):
    def __init__(self, contactManagement):
        b2ContactListener.__init__(self)
        self._contactManagement = contactManagement

    def BeginContact(self, contact):
        thingA = contact.fixtureA.body.userData
        thingB = contact.fixtureB.body.userData
        self._contactManagement(thingA, thingB, 'begin', contact)

    def EndContact(self, contact):
        thingA = contact.fixtureA.body.userData
        thingB = contact.fixtureB.body.userData
        self._contactManagement(thingA, thingB, 'end', contact)

    def PreSolve(self, contact, oldManifold):
        pass

    def PostSolve(self, contact, impulse):
        pass

# class ebQueryCallback(b2QueryCallback):
#     # obsolète
#     def __init__(self):
#         b2QueryCallback.__init__(self)
#         self.objects = []
#
#     def ReportFixture(self, fixture):
#         # remember the associated thing
#         self.objects.append(fixture.body.userData)
#         # continues the query if returns True
#         return True
#
class ebQueryCallback2(b2QueryCallback):
    # obsolète
    def __init__(self, query):
        b2QueryCallback.__init__(self)
        self._query = query

    def ReportFixture(self, fixture):
        # get the associated thing
        thing = fixture.body.userData
        # continues the query if returns True
        return self._query(thing)

class ebRayCastCallback(b2RayCastCallback):
    # travail en cours...
    def __init__(self, raycast):
        b2RayCastCallback.__init__(self)
        self.fixture = None
        self._raycast = raycast

    # Called for each fixture found in the query. You control how the ray proceeds
    # by returning a float that indicates the fractional length of the ray. By returning
    # 0, you set the ray length to zero. By returning the current fraction, you proceed
    # to find the closest point. By returning 1, you continue with the original ray
    # clipping.
    def ReportFixture(self, fixture, point, normal, fraction):
        self.fixture = fixture
        self.point  = b2Vec2(point)
        self.normal = b2Vec2(normal)
        # You will get this error: "TypeError: Swig director type mismatch in output value of type 'float32'"
        # without returning a value
        # get the associated thing
        thing = fixture.body.userData
        return self._raycast(thing, fraction)




from guizero import Drawing
from Box2D import *
from .thing import *
from math import pi,cos,sin

class ebCircle(ebThing):
    def rotation(self,x,y,alpha):
        return cos(alpha)*x+sin(alpha)*y, -sin(alpha)*x+cos(alpha)*y

    def __init__(self, box, x, y, r, **kwargs):
        world  = box.world
        canvas = box.canvas

        # mémorisation de la box
        self.box   = box

        # paramètres
        self._type      = kwargs.pop('type', 'dynamic')

        dens            = kwargs.pop('density', 1)
        fric            = kwargs.pop('friction', 0)
        rest            = kwargs.pop('restitution', 0)

        angle           = kwargs.pop('angle', 0)
        fixedRot        = kwargs.pop('fixedRotation', False)

        bullet          = kwargs.pop('bullet', False)

        # body box2d
        if self._type=='static':
            constructor = world.CreateStaticBody
        elif self._type=='kinematic':
            constructor = world.CreateKinematicBody
        else:
            constructor = world.CreateDynamicBody
        body = constructor(
            position=(x/box.scale, box._height - y/box.scale),
            angle=angle,
            fixedRotation=fixedRot,
            bullet=bullet,
        )

        # fixture circle
        body.CreateCircleFixture(
            radius=r/box.scale,
            density=dens,
            friction=fric,
            restitution=rest,
        )

        # fixture polygone
        # verts = []
        # for i in range(16):
        #     verts.append(self.rotation(r/box.scale,0,(i+1)*2*pi/16))
        # body.CreatePolygonFixture(
        #     vertices=verts,
        #     density=dens,
        #     friction=fric,
        #     restitution=rest,
        # )

        # item canvas oval
        pos = body.position
        x = pos[0]*box.scale
        y = (box._height - pos[1])*box.scale
        r = body.fixtures[0].shape.radius*box.scale
        item = canvas.create_oval(x - r, y - r, x + r, y + r, **kwargs)

        # item canvas polygone
        # shape  = body.fixtures[0].shape
        # coords = [ body.transform * v for v in shape.vertices ]
        # coords = [(c[0]*box.scale, (box._height - c[1])*box.scale) for c in coords]
        # item = canvas.create_polygon(coords, **kwargs)

        # on garde les liens vers ce qui vient d'être créé
        self.body  = body
        body.userData = self
        self.item  = item

    def update_item(self):
        box    = self.box
        body   = self.body
        canvas = box.canvas

        # update pour oval
        pos  = body.position
        x = pos[0]*box.scale
        y = (box._height - pos[1])*box.scale
        r = body.fixtures[0].shape.radius*box.scale
        canvas.coords(self.item, x - r, y - r, x + r, y + r)

        # update pour polygon
        # shape  = body.fixtures[0].shape
        # coords = [ body.transform * v for v in shape.vertices ]
        # new    = []
        # for c in coords:
        #     new += box.toCanvasCoords(*c)
        # for c in coords:
        #     new += [int(c[0]*box.scale), int((box._height - c[1])*box.scale)]
        # canvas.coords(self.item, new)



# import setuptools
# from distutils.core import setup

from setuptools import setup
setup(
      name = 'enisebox',
      packages = ['enisebox'],
      package_data = {},
      version = '0.0.2.2',
      description = 'A lib for making video games',
      author = 'BagEddy42',
      author_email = 'Info@enise.fr',
      keywords = ['game', 'physics'], # arbitrary keywords
      install_requires=[
          'box2d',
          'guizero',
      ],
      classifiers = [],
)
